"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.genCompositeComponentLibraries = exports.runHandleMaterial = void 0;
const path = __importStar(require("path"));
const weapps_core_1 = require("../../weapps-core");
const util_1 = require("../util");
const generate_1 = require("./generate");
const generateCals_1 = require("./generateCals");
const json_schema_defaults_1 = __importDefault(require("json-schema-defaults"));
const lodash_1 = require("lodash");
const style_1 = require("../util/style");
const cals_1 = require("@cloudbase/cals");
const config_1 = require("../config");
async function runHandleMaterial(appBuildDir, dependencies = [], i18nConfig, fileCodeMap, isSandbox, processCssUnit, rpxCalcRoot) {
    await handleCompositeComponent({
        dependencies,
        appBuildDir,
        i18nConfig,
        fileCodeMap,
        isSandbox,
        processCssUnit,
        rpxCalcRoot,
    });
}
exports.runHandleMaterial = runHandleMaterial;
async function handleCompositeComponent({ dependencies, appBuildDir, i18nConfig, fileCodeMap, isSandbox, processCssUnit, rpxCalcRoot, }) {
    const compositeDependencies = dependencies.filter((item) => item.isComposite);
    const materialGroupInfoMap = {};
    dependencies.forEach((item) => (materialGroupInfoMap[item.name] = {
        isComposite: item.isComposite,
        version: item.version,
        entries: item.entries,
        schemaVersion: item.schemaVersion,
    }));
    const componentsInfoMap = await (0, util_1.getComponentsInfo)(path.join(appBuildDir, 'src'), dependencies, fileCodeMap);
    await writeLowCodeFilesForCompositeComp(compositeDependencies, appBuildDir, fileCodeMap, isSandbox, processCssUnit);
    await genCompositeComponentLibraries(compositeDependencies, appBuildDir, materialGroupInfoMap, componentsInfoMap, i18nConfig, fileCodeMap, isSandbox, rpxCalcRoot);
}
async function writeLowCodeFilesForCompositeComp(compositeGroups, appBuildDir, fileMap, isSandbox, processCssUnit) {
    await Promise.all(compositeGroups.map(async (gItem) => {
        const common = await (0, util_1.writeLibCommonRes2file)(gItem, path.join(appBuildDir, 'src', 'libraries', `${gItem.name}@${gItem.version}`, 'libCommonRes'));
        Object.assign(fileMap, common);
        const { compLibCommonResource } = gItem;
        let themeCode = '';
        if (compLibCommonResource) {
            themeCode = `
          ${compLibCommonResource.theme.variable || ''}
          ${compLibCommonResource.class || ''}
          ${compLibCommonResource.theme.class || ''}
        `;
        }
        await Promise.all(gItem.components.map(async (cItem) => {
            // 补齐默认函数
            const lowCodes = (0, cals_1.processRuntimeCodeResources)({ id: '$comp' }, cItem.lowCodes || cItem.codeModules, 'component');
            return lowCodes.map(async (m) => {
                const filePath = path.join(appBuildDir, 'src', 'libraries', `${gItem.name}@${gItem.version}`, 'components', cItem.name, 'lowcode');
                cItem.materialName = gItem.name;
                const fileItem = await writeCode2file(m, filePath, cItem, themeCode, isSandbox, processCssUnit);
                Object.assign(fileMap, fileItem);
                return fileMap;
            }, {});
        }));
        return fileMap;
    }));
    return fileMap;
    async function writeCode2file(mod, lowcodeDir, comp, themeCode, isSandbox, processCssUnit) {
        const pageId = `${comp.name}_${comp.id}`;
        const file = path.join(lowcodeDir, (0, weapps_core_1.getCodeModuleFilePath)(pageId, mod, {
            style: '.css',
        }));
        let codeContent = '';
        if (mod.type === 'style') {
            codeContent = await (0, style_1.processLess)(
            // pageId 作为组件样式的 scope
            `.${(0, weapps_core_1.getCompositedComponentClass)(comp)} { \n${themeCode ? themeCode : ''}\n${mod.code}\n }`, processCssUnit);
        }
        else {
            const importApp = `${
            /* !isSandbox */ false ? `import { app } from '/src/app/global-api';` : config_1.LOWCODE_INJECT_APP_TEMPLATE}\nconst $app = new Proxy({}, { get: function(obj, prop){ return app[prop] }});`;
            codeContent = `${importApp}
      ${mod.code.replace(/\$comp/g, 'this.$WEAPPS_COMP')};`;
        }
        return {
            [file]: {
                code: codeContent,
            },
        };
    }
}
function processWeAppsComponent2Cals(source) {
    let calsData = {};
    const { id: compId, name, componentInstances, meta, dataForm, emitEvents, compConfig, groupName, lowCodes = [], } = source;
    const comps = [];
    for (const id in componentInstances) {
        comps.push((0, cals_1.serializeComponent)({
            component: { id: compId },
        }, {
            ...source.componentInstances[id],
            id,
        }, 'component'));
    }
    if (comps.length > 1) {
        calsData.comp = {
            name,
            type: 'COMPLEX',
            module: config_1.OFFICIAL_LIB_KEY,
            component: 'Container',
            items: comps,
        };
    }
    else {
        calsData = {
            ...comps[0],
            name,
            type: 'COMPLEX',
        };
    }
    calsData.resources = calsData.resources || [];
    calsData.resources.push(...lowCodes.map((item) => {
        return {
            code: item.code,
            name: item.name,
            path: item.path.replace(compId, name),
            codeType: item.type,
            type: 'CODE',
        };
    }));
    // calsData.extra = {
    //   aliasId: compId,
    //   meta,
    //   compConfig,
    //   dataForm,
    //   groupName,
    //   emitEvents,
    // };
    return calsData;
}
async function genCompositeComponentLibraries(dependencies = [], appBuildDir, materialGroupInfoMap = {}, componentsInfoMap, i18nConfig, fileMap, isSandbox, rpxCalcRoot) {
    await Promise.all(dependencies.map(async ({ name, version, components }) => {
        const materialNameVersion = `${name}@${version}`;
        const librariesDir = path.join(appBuildDir, 'src/libraries', materialNameVersion);
        await Promise.all(components.map(async (compItem) => {
            compItem.materialName = name;
            const wrapperClass = (0, weapps_core_1.getCompositedComponentClass)(compItem);
            const cals = processWeAppsComponent2Cals({
                ...compItem,
                groupName: compItem.materialName,
            });
            // const { componentSchema, widgets, dataBinds } = getComponentSchemaString(
            //   {
            //     type: 'object',
            //     // @ts-ignore
            //     properties: readCmpInstances(compItem.componentInstances),
            //   },
            //   true,
            //   componentsInfoMap,
            //   wrapperClass,
            //   'rpx',
            // );
            const { componentSchema, widgets, dataBinds } = (0, generateCals_1.getComponentSchema)({ app: {}, $app: {}, $page: {}, this: {} }, { items: [cals], extra: {} }, componentsInfoMap, wrapperClass, {
                isComposite: true,
                processCssUnit: 'rpx',
                rpxCalcRoot,
            });
            try {
                fileMap = (0, generate_1.generateFilesFromTemplate)({
                    templatesData: {
                        [`pages/composite.tpl.jsx|components/${compItem.name}/index.jsx`]: {
                            // @ts-ignore
                            id: compItem.id,
                            name: compItem.name,
                            defaultProps: (0, json_schema_defaults_1.default)({
                                type: 'object',
                                properties: compItem.dataForm || {},
                            }),
                            compConfig: compItem.compConfig || {},
                            emitEvents: JSON.stringify(compItem.emitEvents.map((evt) => evt.eventName)),
                            // @ts-ignore
                            handlersImports: compItem.lowCodes.filter((codeItem) => codeItem.type === cals_1.ECodeType.HANDLER_FN && codeItem.name !== cals_1.ECodeName.PLACEHOLDER),
                            // @ts-ignore
                            useComponents: (function () {
                                const list = [];
                                // @ts-ignore
                                JSON.stringify(compItem.componentInstances, (key, value) => {
                                    var _a;
                                    if (key === 'xComponent') {
                                        const { moduleName, name } = value;
                                        const compLib = materialGroupInfoMap[moduleName];
                                        let isPlainProps = false;
                                        if (compLib) {
                                            const { schemaVersion = '' } = compLib;
                                            try {
                                                if (Number((_a = schemaVersion.split('.')) === null || _a === void 0 ? void 0 : _a[0]) >= 3) {
                                                    isPlainProps = true;
                                                }
                                            }
                                            catch (e) { }
                                        }
                                        list.push({
                                            moduleName,
                                            name,
                                            key: `${moduleName}:${name}`,
                                            var: (0, lodash_1.upperFirst)((0, util_1.camelcase)(`${moduleName}:${name}`)),
                                            moduleNameVar: (0, util_1.camelcase)(moduleName),
                                            version: (compLib === null || compLib === void 0 ? void 0 : compLib.version) || '',
                                            isComposite: compLib === null || compLib === void 0 ? void 0 : compLib.isComposite,
                                            isPlainProps,
                                            entries: compLib === null || compLib === void 0 ? void 0 : compLib.entries,
                                        });
                                    }
                                    return value;
                                });
                                return (0, lodash_1.uniqBy)(list, 'key');
                            })(),
                            widgets,
                            dataBinds,
                            componentSchema,
                            // @ts-ignore
                            pageListenerInstances: (0, generate_1.getListenersString)(compItem.listeners, true),
                            materialName: name,
                            hasComponentAPI: compItem.lowCodes.some(
                            // @ts-ignore
                            (item) => item.type === 'index'),
                            isSandbox,
                        },
                    },
                    fileCodeMap: fileMap,
                    baseTargetPath: librariesDir,
                });
            }
            catch (e) {
                console.error('>>> template composite.tpl.js', e);
            }
        }));
    }));
    return fileMap;
}
exports.genCompositeComponentLibraries = genCompositeComponentLibraries;
