"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizePageId = exports.generateExpressionAlias = exports.cleanVarName = exports.camelcase = exports.upperFirst = exports.getFileNameByUrl = exports.writeLibCommonRes2file = exports.getYyptConfigInfo = exports.getInputProps = exports.getComponentsInfo = exports.deepDealSchema = exports.JsonToStringWithVariableName = exports.simpleDeepClone = exports.deepDeal = exports.isPlainObject = exports.isArray = exports.getMetaInfoBySourceKey = exports.deepDealComponentSchemaJson = void 0;
const path_1 = __importDefault(require("path"));
const url_1 = __importDefault(require("url"));
const react_schema_renderer_1 = require("@formily/react-schema-renderer");
const declare_1 = require("@cloudbase/weda-client/dist/common/declare");
const acorn_1 = require("acorn");
const acorn_walk_1 = require("acorn-walk");
const utilFunctionConfig = declare_1.formulaData.reduce((map, category) => {
    const list = category.children || [];
    list.forEach((item) => (map[item.id] = true));
    return map;
}, {});
function deepDealComponentSchemaJson(schema, dealFn) {
    if (!schema)
        return;
    dealFn(schema);
    if (schema.properties) {
        Object.keys(schema.properties).forEach((key) => {
            var _a;
            const childSchema = (_a = schema === null || schema === void 0 ? void 0 : schema.properties) === null || _a === void 0 ? void 0 : _a[key];
            if (childSchema) {
                deepDealComponentSchemaJson(childSchema, dealFn);
            }
        });
    }
}
exports.deepDealComponentSchemaJson = deepDealComponentSchemaJson;
function getMetaInfoBySourceKey(sourceKey) {
    if (!sourceKey) {
        return {};
    }
    const [materialName, name] = sourceKey.split(':');
    return {
        materialName,
        name,
        variableName: camelcase(`${materialName}_${name}`),
    };
}
exports.getMetaInfoBySourceKey = getMetaInfoBySourceKey;
function isArray(src) {
    return Object.prototype.toString.call(src) === '[object Array]';
}
exports.isArray = isArray;
function isPlainObject(src) {
    return Object.prototype.toString.call(src) === '[object Object]';
}
exports.isPlainObject = isPlainObject;
function deepDeal(src, reviver) {
    // 对于 数组
    if (isArray(src)) {
        for (let i = 0, len = src.length; i < len; i++) {
            deepDeal(src[i], reviver);
            reviver === null || reviver === void 0 ? void 0 : reviver(i, src[i], src);
        }
    }
    // 对于 Object
    if (isPlainObject(src)) {
        for (const key in src) {
            if (Object.prototype.hasOwnProperty.call(src, key)) {
                // 忽略掉继承属性
                deepDeal(src[key], reviver);
                reviver === null || reviver === void 0 ? void 0 : reviver(key, src[key], src);
            }
        }
    }
}
exports.deepDeal = deepDeal;
function simpleDeepClone(data) {
    return JSON.parse(JSON.stringify(data));
}
exports.simpleDeepClone = simpleDeepClone;
function JsonToStringWithVariableName(copyJson) {
    let variable = JSON.stringify(copyJson, null, 2).replace(/"%%%(.*?)%%%"/g, (match, expression) => {
        return expression.replace(/\\"/g, '"').replace(/\\'/g, "'").replace(/\\r/g, '\r').replace(/\\n/g, '\n');
    });
    return variable;
}
exports.JsonToStringWithVariableName = JsonToStringWithVariableName;
function deepDealSchema(sourceSchema, deal) {
    const fieldSchema = new react_schema_renderer_1.Schema(sourceSchema);
    if (fieldSchema === null || fieldSchema === void 0 ? void 0 : fieldSchema.isObject()) {
        Object.keys(fieldSchema.properties || {}).forEach((key) => {
            var _a, _b;
            const schema = (_a = fieldSchema.properties) === null || _a === void 0 ? void 0 : _a[key];
            const { 'x-props': xProps } = schema || {};
            if (((_b = xProps === null || xProps === void 0 ? void 0 : xProps.data) === null || _b === void 0 ? void 0 : _b._visible) === false) {
                // 暂时不知道 fieldSchema.properties.key 如何删除，暂时使用空Schema替换
                fieldSchema.setProperty(key, new react_schema_renderer_1.Schema({}));
            }
            else if (schema) {
                deepDealSchema(schema, deal);
                deal === null || deal === void 0 ? void 0 : deal(schema, key);
            }
        });
    }
    return fieldSchema.toJSON();
}
exports.deepDealSchema = deepDealSchema;
async function getComponentsInfo(appBuildDir, dependencies, fileCodeMap) {
    const outputObj = {};
    await Promise.all(dependencies.map(async ({ name: materialName, components, isComposite, version }) => {
        if (isComposite) {
            components.forEach((component) => {
                let { lowCodes, componentInstances, ...compItem } = component;
                const sourceKey = `${materialName}:${compItem.name}`;
                outputObj[sourceKey] = {
                    isComposite,
                    ...compItem,
                    _version: version,
                };
            });
        }
        else {
            await Promise.all(components.map(async ({ name, meta, emitEvents, events = undefined }) => {
                outputObj[`${materialName}:${name}`] = {
                    isComposite,
                    ...meta,
                    events,
                    emitEvents,
                    _version: version,
                };
            }));
        }
    }));
    return outputObj;
}
exports.getComponentsInfo = getComponentsInfo;
function getInputProps(componentsMetaMap) {
    const outputObj = {};
    for (let key in componentsMetaMap) {
        let component = componentsMetaMap[key];
        const sourceKey = key;
        if (component.isComposite) {
            let compItem = component;
            Object.keys(compItem.dataForm || {}).forEach((key) => {
                var _a, _b;
                const inputProps = ((_a = compItem.dataForm[key]) === null || _a === void 0 ? void 0 : _a.inputProp) || ((_b = compItem.dataForm[key]) === null || _b === void 0 ? void 0 : _b.syncProp);
                if (inputProps) {
                    outputObj[sourceKey] = {
                        [key]: inputProps,
                        ...(outputObj[sourceKey] || {}),
                    };
                }
            });
        }
        else {
            const inputProps = (component === null || component === void 0 ? void 0 : component.inputProps) || (component === null || component === void 0 ? void 0 : component.syncProps);
            if (inputProps) {
                outputObj[sourceKey] = inputProps;
            }
        }
    }
    return outputObj;
}
exports.getInputProps = getInputProps;
async function getYyptConfigInfo(extraData) {
    const configJson = {
        yyptAppKey: '',
        reportUrl: '',
        stopReport: 'true',
    };
    if (!extraData || !extraData.operationService) {
        extraData = extraData || {};
        extraData.operationService = extraData.operationService || {
            yyptEnabled: true,
        };
    }
    const yyptAppKey = configJson.yyptAppKey || extraData.operationService.extAppId || '';
    const reportUrl = configJson.reportUrl || extraData.operationService.reportUrl || '';
    const stopReport = configJson.stopReport === 'true' || !extraData.operationService.yyptEnabled;
    return {
        yyptAppKey,
        reportUrl,
        stopReport,
    };
}
exports.getYyptConfigInfo = getYyptConfigInfo;
async function writeLibCommonRes2file(gItem, codeDir) {
    var _a, _b, _c, _d;
    const { compLibCommonResource } = gItem;
    const libCommonResFiles = [];
    libCommonResFiles.push({
        path: path_1.default.join(codeDir, `class.less`),
        code: `
        ${((_a = compLibCommonResource === null || compLibCommonResource === void 0 ? void 0 : compLibCommonResource.theme) === null || _a === void 0 ? void 0 : _a.variable) || ''}
        ${(compLibCommonResource === null || compLibCommonResource === void 0 ? void 0 : compLibCommonResource.class) || ''}
        ${((_b = compLibCommonResource === null || compLibCommonResource === void 0 ? void 0 : compLibCommonResource.theme) === null || _b === void 0 ? void 0 : _b.class) || ''}
        `,
    }, {
        path: path_1.default.join(codeDir, 'const.js'),
        code: compLibCommonResource ? (_c = compLibCommonResource === null || compLibCommonResource === void 0 ? void 0 : compLibCommonResource.const) === null || _c === void 0 ? void 0 : _c.code : '',
    }, {
        path: path_1.default.join(codeDir, 'tools.js'),
        code: compLibCommonResource ? (_d = compLibCommonResource === null || compLibCommonResource === void 0 ? void 0 : compLibCommonResource.tools) === null || _d === void 0 ? void 0 : _d.code : '',
    });
    return libCommonResFiles.reduce((ret, item) => {
        ret[item.path] = {
            code: item.code,
        };
        return ret;
    }, {});
}
exports.writeLibCommonRes2file = writeLibCommonRes2file;
/**
 *
 * @param fileUrl 带文件名的url请求地址
 */
function getFileNameByUrl(fileUrl) {
    const parsedUrl = url_1.default.parse(fileUrl);
    const filename = path_1.default.basename((parsedUrl === null || parsedUrl === void 0 ? void 0 : parsedUrl.pathname) || '');
    return filename;
}
exports.getFileNameByUrl = getFileNameByUrl;
function upperFirst(str) {
    return str[0].toUpperCase() + str.slice(1);
}
exports.upperFirst = upperFirst;
function camelcase(str, firstUpperCase = false) {
    str = str.replace(/[:_-]([a-zA-Z])/g, (l) => {
        return l[1].toUpperCase();
    });
    if (firstUpperCase)
        str = str.charAt(0).toUpperCase() + str.slice(1);
    return str;
}
exports.camelcase = camelcase;
function cleanVarName(name) {
    return name.replace(/[@-]/g, '_');
}
exports.cleanVarName = cleanVarName;
function generateExpressionAlias(expression) {
    let imports = '';
    const list = new Set();
    (0, acorn_walk_1.simple)((0, acorn_1.parse)(`(\n${expression}\n)`, { sourceType: 'script', ecmaVersion: 'latest' }), {
        Identifier(node) {
            if (utilFunctionConfig[node.name]) {
                list.add(node.name);
            }
        },
    });
    if (list.size) {
        imports += `const {${Array.from(list).join(',')}} = $app.utils || {};`;
    }
    return imports;
}
exports.generateExpressionAlias = generateExpressionAlias;
/**
 * pageId 名称管理器
 * 因为 pageId 命名为非法
 */
const _MANAGER_PAGE_ID_LIST = [];
const _MANAGER_PAGE_ID_MAP = {};
function normalizePageId(pageId) {
    if (!_MANAGER_PAGE_ID_MAP[pageId]) {
        _MANAGER_PAGE_ID_MAP[pageId] = `$${_MANAGER_PAGE_ID_LIST.length}`;
        _MANAGER_PAGE_ID_LIST.push(pageId);
    }
    return _MANAGER_PAGE_ID_MAP[pageId];
}
exports.normalizePageId = normalizePageId;
