export declare type PX = string;
export declare type Percent = number;
export declare type Color = string;
export declare type URL = string;
import { Properties } from 'csstype';
export declare type CSSProperties = Properties<string | number>;
export interface ICommonStyle {
    size?: ISizeStyle;
    transform?: ITransformStyle;
    text?: ITextStyle;
    margin?: IDistanceStyleWithAuto;
    padding?: IDistanceStyle;
    border?: IBorderStyle;
    background?: IBackgroundStyle;
    position?: IPositionStyle;
    zIndex?: number;
    custom?: ICustomStyle[];
    self?: object;
    display?: string;
    flexConfig?: IFlexConfigStyle;
    boxShadow?: IBoxShadowItemStyle[] | IBoxShadowItemStyle;
}
export interface IPositionStyle {
    position?: 'fixed' | 'absolute' | 'relative' | 'static';
    top?: PX;
    right?: PX | 'auto';
    bottom?: PX;
    left?: PX | 'auto';
}
export interface ISizeStyle {
    autoWidth: boolean;
    width: PX;
    autoHeight: boolean;
    height: PX;
}
export interface ITransformStyle {
    rotate: number;
    opacity: number;
    scale: number;
    radius: PX;
}
export interface ITextStyle {
    color: Color;
    fontSize: PX;
    lineHeight: PX;
    textAlign: 'left' | 'center' | 'right' | 'justify';
    weight: 'lighter' | 'bolder' | 'normal';
    opacity: number;
}
export declare type IBorderWidth = PX | 'inherit' | 'initial' | 'unset' | 'thin' | 'medium' | 'thick';
export interface IBorderStyle {
    type: 'none' | 'solid' | 'dashed';
    color: Color;
    width: IBorderWidth | {
        top?: IBorderWidth;
        right?: IBorderWidth;
        bottom?: IBorderWidth;
        left?: IBorderWidth;
    };
    radius?: PX;
    radiusInfo?: {
        topLeft: string;
        topRight: string;
        bottomRight: string;
        bottomLeft: string;
    };
}
export interface IBackgroundStyle {
    bgType: 'color' | 'image';
    color: Color;
    image: URL;
    size: string;
    repeat: string;
    position?: string;
    positionObj?: {
        left: string;
        top: string;
    };
}
export interface IDistanceStyle {
    top?: PX;
    right?: PX;
    bottom?: PX;
    left?: PX;
}
export interface IDistanceStyleWithAuto {
    top?: PX;
    right?: PX | 'auto';
    bottom?: PX;
    left?: PX | 'auto';
}
export interface ICustomStyle {
    key: string;
    value: string;
}
export interface IFlexConfigStyle {
    flexDirection?: string;
    justifyContent?: string;
    alignItems?: string;
    flexWrap?: string;
}
/**
 * Keyword values
 * box-shadow: none;
 *
 * offset-x | offset-y | color
 * box-shadow: 60px -16px teal;
 *
 * offset-x | offset-y | blur-radius | color
 * box-shadow: 10px 5px 5px black;
 *
 * offset-x | offset-y | blur-radius | spread-radius | color
 * box-shadow: 2px 2px 2px 1px rgba(0, 0, 0, 0.2);
 *
 * inset | offset-x | offset-y | color
 * box-shadow: inset 5em 1em gold;
 *
 * Any number of shadows, separated by commas
 * box-shadow: 3px 3px red, -1em 0 0.4em olive;
 *
 * Global values
 * box-shadow: inherit;
 * box-shadow: initial;
 * box-shadow: revert;
 * box-shadow: revert-layer;
 * box-shadow: unset;
 *
 */
export interface IBoxShadowItemStyle {
    keyWord?: 'none';
    offsetX?: string;
    offsetY?: string;
    blurRadius?: string;
    spreadRadius?: string;
    color?: string;
    global?: 'inherit' | 'initial' | 'revert' | 'revert-layer' | 'unset';
    inset?: boolean;
}
